@@ -3,10 +3,10 @@
  *  Module    : attrib.c
  *  Author    : I. Lea
  *  Created   : 1993-12-01
- *  Updated   : 2015-10-09
+ *  Updated   : 2017-08-06
  *  Notes     : Group attribute routines
  *
- * Copyright (c) 1993-2017 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1993-2019 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -321,6 +321,10 @@ read_attributes_file(
 
 	if ((fp = fopen(file, "r")) != NULL) {
 		scope[0] = '\0';
+		/*
+		 * TODO: use tin_fgets() instead to handle long lines
+		 * (e.g. buggy translation)
+		 */
 		while (fgets(line, (int) sizeof(line), fp) != NULL) {
 			if (line[0] == '\n')
 				continue;
@@ -562,6 +566,19 @@ read_attributes_file(
 							free(tbuf);
 							found = TRUE;
 						}
+						/*
+						 * previous versions has always passed groupname to external
+						 * commands, now we look for %G
+						 *
+						 * 8 == sizeof("sigfile=")
+						 */
+						if (match_string(line, "sigfile=", buf, sizeof(buf) - 8) && buf[0] == '!') {
+							char *newbuf = my_malloc(sizeof(buf) - 8 + 4);
+
+							sprintf(newbuf, "%s %s", buf, "%G");
+							set_attrib(OPT_ATTRIB_SIGFILE, scope, line, newbuf);
+							free(newbuf);
+						}
 						MATCH_BOOLEAN("show_only_unread=", OPT_ATTRIB_SHOW_ONLY_UNREAD_ARTS);
 						MATCH_INTEGER("sort_art_type=", OPT_ATTRIB_SORT_ARTICLE_TYPE, SORT_ARTICLES_BY_LINES_ASCEND);
 						break;
@@ -577,8 +594,14 @@ read_attributes_file(
 
 			if (found)
 				found = FALSE;
-			else
+			else {
+				/* TODO: even without DEBUG? */
 				error_message(1, _(txt_bad_attrib), line);
+#ifdef DEBUG
+				if (debug & (DEBUG_ATTRIB))
+					debug_print_file("ATTRIBUTES", txt_bad_attrib, line);
+#endif /* DEBUG */
+			}
 		}
 		fclose(fp);
 
@@ -1325,7 +1348,7 @@ write_attributes_file(
 
 	/*
 	 * determine the file offset
-	 * this is nesessary because a changed locale setting
+	 * this is necessary because a changed locale setting
 	 * may lead to an invalid offset
 	 */
 	if ((fpos = ftell(fp)) <= 0) {
@@ -1528,7 +1551,6 @@ write_attributes_file(
 		rename_file(new_file, file);
 
 	free(new_file);
-	return;
 }
 
 
