@@ -3,10 +3,10 @@
  *  Module    : config.c
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2017-08-23
+ *  Updated   : 2018-11-22
  *  Notes     : Configuration file routines
  *
- * Copyright (c) 1991-2018 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2019 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -1631,15 +1631,11 @@ match_string(
 	char *dst,
 	size_t dstlen)
 {
-	char *ptr;
 	size_t patlen = strlen(pat);
 
-	if (STRNCMPEQ(line, pat, patlen) && (strlen(line) > patlen /* + 1 */)) {
-		if (dst != NULL && dstlen >= 1) {
-			strncpy(dst, &line[patlen], dstlen);
-			if ((ptr = strrchr(dst, '\n')) != NULL)
-				*ptr = '\0';
-		}
+	if (STRNCMPEQ(line, pat, patlen) && (strlen(line) > patlen)) {
+		if (dst != NULL && dstlen >= 1)
+			my_strncpy(dst, &line[patlen], dstlen - 1);
 		return TRUE;
 	}
 	return FALSE;
@@ -1658,13 +1654,14 @@ match_item(
 	char *nline = my_strdup(line);
 	size_t patlen = strlen(pat);
 
-	nline[strlen(nline) - 1] = '\0'; /* remove tailing \n */
+	if ((ptr = strchr(nline, '\n')) != NULL) /* terminate on \n */
+		*ptr = '\0';
 
 	if (!strcasecmp(nline, pat)) {
-		strncpy(dst, &nline[patlen], dstlen);
-		if ((ptr = strrchr(dst, '\n')) != NULL)
-			*ptr = '\0';
-
+		if (dst != NULL) {
+			strncpy(dst, &nline[patlen], dstlen);
+			dst[dstlen ? (dstlen - 1) : 0] = '\0';
+		}
 		free(nline);
 		return TRUE;
 	}
@@ -1779,7 +1776,7 @@ static t_bool
 rc_update(
 	FILE *fp)
 {
-	char buf[1024];
+	char buf[LEN];
 	int show_info = 1;
 	t_bool auto_bcc = FALSE;
 	t_bool auto_cc = FALSE;
@@ -2022,7 +2019,7 @@ static t_bool
 rc_post_update(
 	FILE *fp)
 {
-	char buf[1024];
+	char buf[LEN];
 	int groupname_max_length = 0;
 
 	if (!fp)
@@ -2126,8 +2123,8 @@ read_server_config(
 	joinpath(local_newsgroups_file, sizeof(local_newsgroups_file), serverdir, NEWSGROUPS_FILE);
 	if ((fp = fopen(file, "r")) == NULL)
 		return;
-	while (NULL != (line = tin_fgets(fp, FALSE))) {
-		if (('#' == *line) || ('\0' == *line))
+	while ((line = tin_fgets(fp, FALSE)) != NULL) {
+		if ((*line == '#') || (*line == '\0'))
 			continue;
 
 		if (match_string(line, "last_newnews=", newnews_info, sizeof(newnews_info))) {
@@ -2140,12 +2137,9 @@ read_server_config(
 			continue;
 		}
 		if (match_string(line, "version=", NULL, 0)) {
-			if (RC_CHECK != upgrade)
-				/* ignore duplicate version lines; last match counts */
+			if (upgrade != RC_CHECK) /* ignore duplicate version lines; last match counts */
 				continue;
-			upgrade = check_upgrade(line, "version=", SERVERCONFIG_VERSION);
-			if (RC_IGNORE == upgrade)
-				/* Expected version number; nothing to do -> continue */
+			if ((upgrade = check_upgrade(line, "version=", SERVERCONFIG_VERSION)) == RC_IGNORE) /* Expected version number; nothing to do -> continue */
 				continue;
 
 			/* Nothing to do yet for RC_UPGRADE and RC_DOWNGRADE */
@@ -2185,8 +2179,8 @@ write_server_config(
 	if ((no_write || post_article_and_exit || post_postponed_and_exit) && file_size(file) != -1L)
 		return;
 
-	if (-1 == stat(serverdir, &statbuf)) {
-		if (-1 == my_mkdir(serverdir, (mode_t) (S_IRWXU)))
+	if (stat(serverdir, &statbuf) == -1) {
+		if (my_mkdir(serverdir, (mode_t) (S_IRWXU)) == -1)
 			/* Can't create directory TODO: Add error handling */
 			return;
 	}
