@@ -3,10 +3,10 @@
  *  Module    : cook.c
  *  Author    : J. Faultless
  *  Created   : 2000-03-08
- *  Updated   : 2016-02-28
+ *  Updated   : 2018-11-22
  *  Notes     : Split from page.c
  *
- * Copyright (c) 2000-2018 Jason Faultless <jason@altarstone.com>
+ * Copyright (c) 2000-2019 Jason Faultless <jason@altarstone.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -457,7 +457,7 @@ process_text_body_part(
 				/*
 				 * FIXME: Some code in cook.c expects a '\n' at the end
 				 * of the line. As tin_fgets() strips trailing '\n', re-add it.
-				 * This should problably be fixed in that other code.
+				 * This should probably be fixed in that other code.
 				 */
 				strcat(line, "\n");
 
@@ -491,7 +491,7 @@ process_text_body_part(
 			len_blank = 1;
 			tmpline = line;
 			/* check if line contains only whitespace */
-			while ((' ' == *tmpline) || ('\t' == *tmpline)) {
+			while ((*tmpline == ' ') || (*tmpline == '\t')) {
 				len_blank++;
 				tmpline++;
 			}
@@ -836,7 +836,7 @@ cook_article(
 	static const char *struct_header[] = {
 		"Approved: ", "From: ", "Originator: ",
 		"Reply-To: ", "Sender: ", "X-Cancelled-By: ", "X-Comment-To: ",
-		"X-Submissions-To: ", "To: ", "Cc: ", "Bcc: ", "X-Originator: ", 0 };
+		"X-Submissions-To: ", "To: ", "Cc: ", "Bcc: ", "X-Originator: ", NULL };
 
 	art = artinfo;				/* Global saves lots of passing artinfo around */
 
