@@ -3,10 +3,10 @@
  *  Module    : inews.c
  *  Author    : I. Lea
  *  Created   : 1992-03-17
- *  Updated   : 2017-08-13
+ *  Updated   : 2018-11-22
  *  Notes     : NNTP built in version of inews
  *
- * Copyright (c) 1991-2018 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2019 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -398,14 +398,14 @@ submit_news_file(
 	rfc15211522_encode(name, txt_mime_encodings[(group ? group->attribute->post_mime_encoding : tinrc.post_mime_encoding)], group, (group ? group->attribute->post_8bit_header : tinrc.post_8bit_header), ismail);
 
 #ifdef NNTP_INEWS
-	if (read_news_via_nntp && !read_saved_news && 0 == strcasecmp(tinrc.inews_prog, INTERNAL_CMD))
+	if (read_news_via_nntp && !read_saved_news && !strcasecmp(tinrc.inews_prog, INTERNAL_CMD))
 		ret_code = submit_inews(name, group, a_message_id);
 	else
 #endif /* NNTP_INEWS */
 		{
 #ifdef M_UNIX
 			/* use tinrc.inews_prog or 'inewsdir/inews -h' 'inews -h' */
-			if (0 != strcasecmp(tinrc.inews_prog, INTERNAL_CMD))
+			if (strcasecmp(tinrc.inews_prog, INTERNAL_CMD))
 				STRCPY(buf, tinrc.inews_prog);
 			else {
 				if (*inewsdir)
@@ -422,7 +422,7 @@ submit_news_file(
 			ret_code = invoke_cmd(buf);
 
 #ifdef NNTP_INEWS
-			if (!ret_code && read_news_via_nntp && !read_saved_news && 0 != strcasecmp(tinrc.inews_prog, INTERNAL_CMD)) {
+			if (!ret_code && read_news_via_nntp && !read_saved_news && strcasecmp(tinrc.inews_prog, INTERNAL_CMD)) {
 				if (prompt_yn(_(txt_post_via_builtin_inews), TRUE)) {
 					ret_code = submit_inews(name, group, a_message_id);
 					if (ret_code) {
@@ -481,7 +481,7 @@ sender_needed(
 	from_at_pos = strchr(from_addr, '@');
 	if ((sender_at_pos = strchr(sender_addr, '@')))
 		sender_dot_pos = strchr(sender_at_pos, '.');
-	else /* this case is catched by the gnksa_do_check_from() code above; anyway ... */
+	else /* this case is caught by the gnksa_do_check_from() code above; anyway ... */
 		return -2;
 
 	if (from_at_pos == NULL || sender_dot_pos == NULL) /* as we build From and check Sender above this shouldn't happen at all */
