@@ -3,10 +3,10 @@
  *  Module    : misc.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2017-09-18
+ *  Updated   : 2018-11-23
  *  Notes     :
  *
- * Copyright (c) 1991-2018 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2019 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -248,7 +248,7 @@ backup_file(
  * prefix (= quote_chars), initials of the articles author
  * with_sig is set if the signature should be quoted
  *
- * TODO: rewrite from scratch, the code is awfull
+ * TODO: rewrite from scratch, the code is awful
  */
 void
 copy_body(
@@ -459,7 +459,7 @@ invoke_ispell(
 		fputs(buf, fp_head);
 		if (buf[0] == '\n' || buf[0] == '\r') {
 			fclose(fp_head);
-			fp_head = 0;
+			fp_head = NULL;
 			break;
 		}
 	}
@@ -919,7 +919,7 @@ mail_check(
 
 	mailbox_name = get_val("MAIL", mailbox);
 
-	if (mailbox_name != 0 && stat(mailbox_name, &buf) >= 0) {
+	if (mailbox_name != NULL && stat(mailbox_name, &buf) >= 0) {
 		if ((int) (buf.st_mode & S_IFMT) == (int) S_IFDIR) { /* maildir setup */
 			char *maildir_box;
 			size_t maildir_box_len = strlen(mailbox_name) + strlen(MAILDIR_NEW) + 2;
@@ -1127,7 +1127,7 @@ show_color_status(
  * Check for lock file to stop multiple copies of tin -u running and if it
  * does not exist create it so this is the only copy running
  *
- * FIXME: get ridd of hardcoded pid-length as pid_t might be long
+ * FIXME: get rid of hard coded pid-length as pid_t might be long
  */
 void
 create_index_lock_file(
@@ -1180,7 +1180,7 @@ strfquote(
 	size_t maxsize,
 	char *format)
 {
-	char *endp = s + maxsize;
+	char *endp;
 	char *start = s;
 	char tbuf[LEN];
 	int i, j;
@@ -1192,6 +1192,7 @@ strfquote(
 	if (strchr(format, '%') == NULL && strlen(format) + 1 >= maxsize)
 		return 0;
 
+	endp = s + maxsize;
 	for (; *format && s < endp - 1; format++) {
 		tbuf[0] = '\0';
 
@@ -1339,7 +1340,7 @@ strfeditor(
 	size_t maxsize,
 	char *format)
 {
-	char *endp = s + maxsize;
+	char *endp;
 	char *start = s;
 	char tbuf[PATH_LEN];
 	int i;
@@ -1350,6 +1351,7 @@ strfeditor(
 	if (strchr(format, '%') == NULL && strlen(format) + 1 >= maxsize)
 		return 0;
 
+	endp = s + maxsize;
 	for (; *format && s < endp - 1; format++) {
 		tbuf[0] = '\0';
 
@@ -1493,11 +1495,11 @@ _strfpath(
 	struct t_group *group,
 	t_bool expand_all)
 {
-	char *endp = str + maxsize;
-	const char *startp = format;
+	char *endp;
+	char *envptr;
 	char defbuf[PATH_LEN];
 	char tbuf[PATH_LEN];
-	char *envptr;
+	const char *startp = format;
 	int i;
 	struct passwd *pwd;
 	t_bool is_mailbox = FALSE;
@@ -1508,6 +1510,7 @@ _strfpath(
 	if (strlen(format) + 1 >= maxsize)
 		return 0;
 
+	endp = str + maxsize;
 	for (; *format && str < endp - 1; format++) {
 		tbuf[0] = '\0';
 
@@ -1783,7 +1786,7 @@ strfmailer(
 	size_t maxsize,
 	const char *format)
 {
-	char *endp = dest + maxsize;
+	char *endp;
 	char *start = dest;
 	char tbuf[PATH_LEN];
 	int quote_area = no_quote;
@@ -1798,9 +1801,9 @@ strfmailer(
 
 	/*
 	 * TODO: shouldn't we better check for no % OR format > maxsize?
-	 *       as no replacemnt doesn't make sense (hardcoded To, Subject
-	 *       and filename) and the resulting string usuly is longer after
-	 *       replacemnts were done (nobody uses enough %% to make the
+	 *       as no replacement doesn't make sense (hard coded To, Subject
+	 *       and filename) and the resulting string usually is longer after
+	 *       replacements were done (nobody uses enough %% to make the
 	 *       result shorter than the input).
 	 */
 	if (strchr(format, '%') == NULL && strlen(format) + 1 >= maxsize)
@@ -1810,6 +1813,7 @@ strfmailer(
 	 * walk through format until end of format or end of available space
 	 * and replace place holders
 	 */
+	endp = dest + maxsize;
 	for (; *format && dest < endp - 1; format++) {
 		tbuf[0] = '\0';
 
@@ -1893,7 +1897,7 @@ strfmailer(
 
 				case 'S':	/* Subject */
 					/* don't MIME encode Subject if using external mail client */
-					if (INTERACTIVE_NONE != tinrc.interactive_mailer)
+					if (tinrc.interactive_mailer != INTERACTIVE_NONE)
 						strncpy(tbuf, escape_shell_meta(subject, quote_area), sizeof(tbuf) - 1);
 					else {
 #ifdef CHARSET_CONVERSION
@@ -1910,7 +1914,7 @@ strfmailer(
 
 				case 'T':	/* To */
 					/* don't MIME encode To if using external mail client */
-					if (INTERACTIVE_NONE != tinrc.interactive_mailer)
+					if (tinrc.interactive_mailer != INTERACTIVE_NONE)
 						strncpy(tbuf, escape_shell_meta(to, quote_area), sizeof(tbuf) - 1);
 					else {
 #ifdef CHARSET_CONVERSION
@@ -1927,7 +1931,7 @@ strfmailer(
 
 				case 'U':	/* User */
 					/* don't MIME encode User if using external mail client */
-					if (INTERACTIVE_NONE != tinrc.interactive_mailer)
+					if (tinrc.interactive_mailer != INTERACTIVE_NONE)
 						strncpy(tbuf, userid, sizeof(tbuf) - 1);
 					else {
 #ifdef CHARSET_CONVERSION
@@ -3000,7 +3004,7 @@ gnksa_dequote_plainphrase(
 
 					case '=':
 						*(wpos++) = *(rpos++);
-						if ('?' == *rpos) {
+						if (*rpos == '?') {
 							state = 2;
 							*(wpos++) = *(rpos++);
 						} else
@@ -3111,7 +3115,7 @@ gnksa_dequote_plainphrase(
 				switch (*rpos) {
 					case '?':
 						*(wpos++) = *(rpos++);
-						if ('=' == *rpos) {
+						if (*rpos == '=') {
 							state = initialstate;
 							*(wpos++) = *(rpos++);
 						} else
@@ -3139,7 +3143,7 @@ gnksa_dequote_plainphrase(
 
 					case '=':
 						*(wpos++) = *(rpos++);
-						if ('?' == *rpos) {
+						if (*rpos == '?') {
 							state = 2;
 							*(wpos++) = *(rpos++);
 						} else
@@ -3180,12 +3184,12 @@ gnksa_check_domain_literal(
 	x1 = x2 = x3 = x4 = 666;
 	term = '\0';
 
-	if ('[' == *domain) { /* literal bracketed */
+	if (*domain == '[') { /* literal bracketed */
 		n = sscanf(domain, "[%u.%u.%u.%u%c", &x1, &x2, &x3, &x4, &term);
-		if (5 != n)
+		if (n != 5)
 			return GNKSA_BAD_DOMAIN_LITERAL;
 
-		if (']' != term)
+		if (term != ']')
 			return GNKSA_BAD_DOMAIN_LITERAL;
 
 	} else { /* literal not bracketed */
@@ -3194,25 +3198,25 @@ gnksa_check_domain_literal(
 #else
 		n = sscanf(domain, "%u.%u.%u.%u%c", &x1, &x2, &x3, &x4, &term);
 		/* there should be no terminating character present */
-		if (4 != n)
+		if (n != 4)
 			return GNKSA_BAD_DOMAIN_LITERAL;
 #endif /* REQUIRE_BRACKETS_IN_DOMAIN_LITERAL */
 	}
 
 	/* check ip parts for legal range */
-	if ((255 < x1) || (255 < x2) || (255 < x3) || (255 < x4))
+	if ((x1 > 255) || (x2 > 255) || (x3 > 255) || (x4 > 255))
 		return GNKSA_BAD_DOMAIN_LITERAL;
 
 	/* check for private ip or localhost - see RFC 5735, RFC 5737 */
 	if ((!disable_gnksa_domain_check)
-	    && ((0 == x1)				/* local network */
-		|| (10 == x1)				/* private class A */
-		|| ((172 == x1) && (16 == (x2 & 0xf0)))	/* private /12 */
-		|| ((192 == x1) && (168 == x2))		/* private class B */
-		|| ((192 == x1) && (0 == x2) && (2 == x3)) /* TEST NET-1 */
-		|| ((198 == x1) && (51 == x2) && (100 == x3)) /* TEST NET-2 */
-		|| ((203 == x1) && (0 == x2) && (113 == x3)) /* TEST NET-3 */
-		|| (127 == x1)))			/* loopback */
+	    && ((x1 == 0)				/* local network */
+		|| (x1 == 10)				/* private class A */
+		|| ((x1 == 172) && ((x2 & 0xf0) == 16))	/* private /12 */
+		|| ((x1 == 192) && (x2 == 168))		/* private class B */
+		|| ((x1 == 192) && (x2 == 0) && (x3 == 2)) /* TEST NET-1 */
+		|| ((x1 == 198) && (x2 == 51) && (x3 == 100)) /* TEST NET-2 */
+		|| ((x1 == 203) && (x2 == 0) && (x3 == 113)) /* TEST NET-3 */
+		|| (x1 == 127)))			/* loopback */
 		return GNKSA_LOCAL_DOMAIN_LITERAL;
 
 	return GNKSA_OK;
@@ -3229,16 +3233,15 @@ gnksa_check_domain(
 	int result;
 
 	/* check for domain literal */
-	if ('[' == *domain) /* check value of domain literal */
+	if (*domain == '[') /* check value of domain literal */
 		return gnksa_check_domain_literal(domain);
 
 	/* check for leading or trailing dot */
-	if (('.' == *domain) || ('.' == *(domain + strlen(domain) - 1)))
+	if ((*domain == '.') || (*(domain + strlen(domain) - 1) == '.'))
 		return GNKSA_ZERO_LENGTH_LABEL;
 
 	/* look for TLD start */
-	aux = strrchr(domain, '.');
-	if (NULL == aux)
+	if ((aux = strrchr(domain, '.')) == NULL)
 		return GNKSA_SINGLE_DOMAIN;
 
 	aux++;
@@ -3247,7 +3250,7 @@ gnksa_check_domain(
 	switch ((int) strlen(aux)) {
 		case 1:
 			/* no numeric components allowed */
-			if (('0' <= *aux) && ('9' >= *aux))
+			if ((*aux >= '0') && (*aux <= '9'))
 				return gnksa_check_domain_literal(domain);
 
 			/* single letter TLDs do not exist */
@@ -3257,12 +3260,12 @@ gnksa_check_domain(
 
 		case 2:
 			/* no numeric components allowed */
-			if (('0' <= *aux) && ('9' >= *aux)
-			    && ('0' <= *(aux + 1)) && ('9' >= *(aux + 1)))
+			if ((*aux >= '0') && (*aux <= '9')
+			    && (*(aux + 1) >= '0') && (*(aux + 1) <= '9'))
 				return gnksa_check_domain_literal(domain);
 
-			if (('a' <= *aux) && ('z' >= *aux)
-			    && ('a' <= *(aux + 1)) && ('z' >= *(aux + 1))) {
+			if ((*aux >= 'a') && (*aux <= 'z')
+			    && (*(aux + 1) >= 'a') && (*(aux + 1) <= 'z')) {
 				i = ((*aux - 'a') * 26) + (*(aux + 1)) - 'a';
 				if (!gnksa_country_codes[i])
 					return GNKSA_UNKNOWN_DOMAIN;
@@ -3272,9 +3275,9 @@ gnksa_check_domain(
 
 		case 3:
 			/* no numeric components allowed */
-			if (('0' <= *aux) && ('9' >= *aux)
-			    && ('0' <= *(aux + 1)) && ('9' >= *(aux + 1))
-			    && ('0' <= *(aux + 2)) && ('9' >= *(aux + 2)))
+			if ((*aux >= '0') && (*aux <= '9')
+			    && (*(aux + 1) >= '0') && (*(aux + 1) <= '9')
+			    && (*(aux + 2) >= '0') && (*(aux + 2) <= '9'))
 				return gnksa_check_domain_literal(domain);
 			/* FALLTHROUGH */
 		default:
@@ -3286,7 +3289,7 @@ gnksa_check_domain(
 			}
 			if (disable_gnksa_domain_check)
 				result = GNKSA_OK;
-			if (GNKSA_OK != result)
+			if (result != GNKSA_OK)
 				return result;
 			break;
 	}
@@ -3294,18 +3297,18 @@ gnksa_check_domain(
 	/* check for illegal labels */
 	last = domain;
 	for (aux = domain; *aux; aux++) {
-		if ('.' == *aux) {
+		if (*aux == '.') {
 			if (aux - last - 1 > 63)
 				return GNKSA_ILLEGAL_LABEL_LENGTH;
 
-			if ('.' == *(aux + 1))
+			if (*(aux + 1) == '.')
 				return GNKSA_ZERO_LENGTH_LABEL;
 
-			if (('-' == *(aux + 1)) || ('-' == *(aux - 1)))
+			if ((*(aux + 1) == '-') || (*(aux - 1) == '-'))
 				return GNKSA_ILLEGAL_LABEL_HYPHEN;
 
 #ifdef ENFORCE_RFC1034
-			if (('0' <= *(aux + 1)) && ('9' >= *(aux + 1)))
+			if ((*(aux + 1) >= '0') && (*(aux + 1) <= '9'))
 				return GNKSA_ILLEGAL_LABEL_BEGNUM;
 #endif /* ENFORCE_RFC1034 */
 			last = aux;
@@ -3336,11 +3339,11 @@ gnksa_check_localpart(
 		return GNKSA_LOCALPART_MISSING;
 
 	/* check for zero-length domain parts */
-	if (('.' == *localpart) || ('.' == *(localpart + strlen(localpart) -1)))
+	if ((*localpart == '.') || (*(localpart + strlen(localpart) - 1) == '.'))
 		return GNKSA_ZERO_LENGTH_LOCAL_WORD;
 
 	for (aux = localpart; *aux; aux++) {
-		if (('.' == *aux) && ('.' == *(aux + 1)))
+		if ((*aux == '.') && (*(aux + 1) == '.'))
 			return GNKSA_ZERO_LENGTH_LOCAL_WORD;
 	}
 
@@ -3378,7 +3381,7 @@ gnksa_split_from(
 
 	/* skip trailing whitespace */
 	addr_end = work + strlen(work) - 1;
-	while (addr_end >= work && (' ' == *addr_end || '\t' == *addr_end))
+	while (addr_end >= work && (*addr_end == ' ' || *addr_end == '\t'))
 		addr_end--;
 
 	if (addr_end < work) {
@@ -3389,7 +3392,7 @@ gnksa_split_from(
 	*(addr_end + 1) = '\0';
 	*(addr_end + 2) = '\0';
 
-	if ('>' == *addr_end) {
+	if (*addr_end == '>') {
 		/* route-address used */
 		*addrtype = GNKSA_ADDRTYPE_ROUTE;
 
@@ -3398,7 +3401,7 @@ gnksa_split_from(
 		while (('<' != *addr_begin) && (addr_begin > work))
 			addr_begin--;
 
-		if ('<' != *addr_begin) /* syntax error in mail address */
+		if (*addr_begin != '<') /* syntax error in mail address */
 			return GNKSA_LANGLE_MISSING;
 
 		/* copy route address */
@@ -3410,10 +3413,10 @@ gnksa_split_from(
 		addr_begin = work;
 
 		/* strip surrounding whitespace */
-		while (addr_end >= work && (' ' == *addr_end || '\t' == *addr_end))
+		while (addr_end >= work && (*addr_end == ' '|| *addr_end == '\t'))
 			addr_end--;
 
-		while ((' ' == *addr_begin) || ('\t' == *addr_begin))
+		while ((*addr_begin == ' ') || (*addr_begin == '\t'))
 			addr_begin++;
 
 		*++addr_end = '\0';
@@ -3426,12 +3429,12 @@ gnksa_split_from(
 		/* get address part */
 		/* skip leading whitespace */
 		addr_begin = work;
-		while ((' ' == *addr_begin) || ('\t' == *addr_begin))
+		while ((*addr_begin == ' ') || (*addr_begin == '\t'))
 			addr_begin++;
 
 		/* scan forward to next whitespace or null */
 		addr_end = addr_begin;
-		while ((' ' != *addr_end) && ('\t' != *addr_end) && (*addr_end))
+		while ((*addr_end != ' ') && (*addr_end != '\t') && (*addr_end))
 			addr_end++;
 
 		*addr_end = '\0';
@@ -3442,19 +3445,19 @@ gnksa_split_from(
 		addr_begin = addr_end + 1;
 		addr_end = addr_begin + strlen(addr_begin) -1;
 		/* strip surrounding whitespace */
-		while ((' ' == *addr_end) || ('\t' == *addr_end))
+		while ((*addr_end == ' ') || (*addr_end == '\t'))
 			addr_end--;
 
-		while ((' ' == *addr_begin) || ('\t' == *addr_begin))
+		while ((*addr_begin == ' ') || (*addr_begin == '\t'))
 			addr_begin++;
 
 		/* any realname at all? */
 		if (*addr_begin) {
 			/* check for parentheses */
-			if ('(' != *addr_begin)
+			if (*addr_begin != '(')
 				return GNKSA_LPAREN_MISSING;
 
-			if (')' != *addr_end)
+			if (*addr_end != ')')
 				return GNKSA_RPAREN_MISSING;
 
 			/* copy realname */
@@ -3464,7 +3467,7 @@ gnksa_split_from(
 	}
 
 	/*
-	 * if we allow <> as From: we must disallow <> as Mesage-ID,
+	 * if we allow <> as From: we must disallow <> as Message-ID,
 	 * see code in post.c:check_article_to_be_posted()
 	 */
 #if 0
@@ -3502,7 +3505,7 @@ gnksa_do_check_from(
 
 	/* split from */
 	code = gnksa_split_from(from, address, realname, &addrtype);
-	if ('\0' == *address) /* address missing or not extractable */
+	if (*address == '\0') /* address missing or not extractable */
 		return code;
 
 #ifdef DEBUG
@@ -3513,7 +3516,7 @@ gnksa_do_check_from(
 	/* parse address */
 	addr_begin = strrchr(address, '@');
 
-	if (NULL != addr_begin) {
+	if (addr_begin != NULL) {
 		/* temporarily terminate string at separator position */
 		*addr_begin++ = '\0';
 
@@ -3525,12 +3528,12 @@ gnksa_do_check_from(
 		/* convert FQDN part to lowercase */
 		str_lwr(addr_begin);
 
-		if (GNKSA_OK != (result = gnksa_check_domain(addr_begin))
-		    && (GNKSA_OK == code)) /* error detected */
+		if ((result = gnksa_check_domain(addr_begin)) != GNKSA_OK
+		    && (code == GNKSA_OK)) /* error detected */
 			code = result;
 
-		if (GNKSA_OK != (result = gnksa_check_localpart(address))
-		    && (GNKSA_OK == code)) /* error detected */
+		if ((result = gnksa_check_localpart(address)) != GNKSA_OK
+		    && (code == GNKSA_OK)) /* error detected */
 			code = result;
 
 		/* restore separator character */
@@ -3543,15 +3546,15 @@ gnksa_do_check_from(
 #endif /* DEBUG */
 
 	/* check realname */
-	if (GNKSA_OK != (result = gnksa_dequote_plainphrase(realname, decoded, addrtype))) {
-		if (GNKSA_OK == code) /* error detected */
+	if ((result = gnksa_dequote_plainphrase(realname, decoded, addrtype)) != GNKSA_OK) {
+		if (code == GNKSA_OK) /* error detected */
 			code = result;
 	} else	/* copy dequoted realname to result variable */
 		strcpy(realname, decoded);
 
 #ifdef DEBUG
 	if (debug & DEBUG_MISC) { /* TODO: dump to a file instead of wait_message() */
-		if (GNKSA_OK != code)
+		if (code != GNKSA_OK)
 			wait_message(2, "From:=[%s], GNKSA=[%d]", from, code);
 		else
 			wait_message(0, "GNKSA=[%d]", code);
@@ -3610,7 +3613,7 @@ strip_line(
 
 #if defined(CHARSET_CONVERSION) || (defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE))
 /*
- * 'check' a given UTF-8 strig and '?'-out illegal sequences
+ * 'check' a given UTF-8 string and '?'-out illegal sequences
  * TODO: is this check complete?
  *
  * UTF-8           = ASCII / UTF-8-non-ascii
@@ -3757,30 +3760,37 @@ idna_decode(
 	char *in)
 {
 	char *out = my_strdup(in);
+
+	/* decoding needed? */
+	if (!strstr(in, "xn--"))
+		return out;
+
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 /* IDNA 2008 */
 #	if defined(HAVE_LIBIDNKIT) && defined(HAVE_IDN_DECODENAME)
-	idn_result_t res;
-	char *q, *r = NULL;
+	{
+		idn_result_t res;
+		char *q, *r = NULL;
 
-	if ((q = strrchr(out, '@'))) {
-		q++;
-		r = strrchr(in, '@');
-		r++;
-	} else {
-		r = in;
-		q = out;
-	}
-	if ((res = idn_decodename(IDN_DECODE_LOOKUP, r, q, out + strlen(out) - q + 1)) == idn_success)
-		return out;
-	else { /* IDNA 2008 failed, try again with IDNA 2003 if available */
-		free(out);
-		out = my_strdup(in);
-	}
+		if ((q = strrchr(out, '@'))) {
+			q++;
+			r = strrchr(in, '@');
+			r++;
+		} else {
+			r = in;
+			q = out;
+		}
+		if ((res = idn_decodename(IDN_DECODE_LOOKUP, r, q, out + strlen(out) - q + 1)) == idn_success)
+			return out;
+		else { /* IDNA 2008 failed, try again with IDNA 2003 if available */
+			free(out);
+			out = my_strdup(in);
+		}
 #		ifdef DEBUG
-	if (debug & DEBUG_MISC)
-		wait_message(2, "idn_decodename(%s): %s", r, idn_result_tostring(res));
+		if (debug & DEBUG_MISC)
+			wait_message(2, "idn_decodename(%s): %s", r, idn_result_tostring(res));
 #		endif /* DEBUG */
+	}
 #	endif /* HAVE_LIBIDNKIT && HAVE_IDN_DECODENAME */
 
 /* IDNA 2003 */
@@ -3826,7 +3836,7 @@ idna_decode(
 #		if defined(HAVE_LIBIDN) && defined(HAVE_IDNA_TO_UNICODE_LZLZ)
 	if (stringprep_check_version("0.3.0")) {
 		char *t, *s = NULL;
-		int rs = IDNA_SUCCESS;
+		int rs;
 
 		if ((t = strrchr(out, '@')))
 			t++;
@@ -4098,11 +4108,11 @@ tin_version_info(
 #else
 			"-REQUIRE_BRACKETS_IN_DOMAIN_LITERAL "
 #endif /* REQUIRE_BRACKETS_IN_DOMAIN_LITERAL */
-#ifdef FOLLOW_USEFOR_DRAFT
-			"+FOLLOW_USEFOR_DRAFT"
+#ifdef ALLOW_FWS_IN_NEWSGROUPLIST
+			"+ALLOW_FWS_IN_NEWSGROUPLIST"
 #else
-			"-FOLLOW_USEFOR_DRAFT"
-#endif /* FOLLOW_USEFOR_DRAFT */
+			"-ALLOW_FWS_IN_NEWSGROUPLIST"
+#endif /* ALLOW_FWS_IN_NEWSGROUPLIST */
 			"\n");
 	wlines += 11;
 	fflush(fp);
