@@ -3,10 +3,10 @@
  *  Module    : newsrc.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2017-05-23
+ *  Updated   : 2018-11-23
  *  Notes     : ArtCount = (ArtMax - ArtMin) + 1  [could have holes]
  *
- * Copyright (c) 1991-2018 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2019 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -72,7 +72,7 @@ static void print_bitmap_seq(FILE *fp, struct t_group *group);
  * accordingly. Bogus groups will _not_ be subscribed to as a design
  * principle.
  *
- * Returns the numer of lines read(useful for a check newsrc >= oldnewsrc)
+ * Returns the number of lines read(useful for a check newsrc >= oldnewsrc)
  * 	< 0 error
  * 	>=0 number of lines read
  */
@@ -343,7 +343,7 @@ auto_subscribe_groups(
 	/* TODO: test me! */
 	while ((ptr = tin_fgets(fp_subs, FALSE)) != NULL) {
 		if (ptr[0] != '#') {
-			if (group_find(ptr, FALSE) != 0)
+			if (group_find(ptr, FALSE) != NULL)
 				fprintf(fp_newsrc, "%s:\n", ptr);
 		}
 	}
@@ -384,8 +384,8 @@ backup_newsrc(
 	joinpath(dirbuf, sizeof(dirbuf), rcdir, filebuf);
 	joinpath(filebuf, sizeof(filebuf), dirbuf, OLDNEWSRC_FILE);
 
-	if (-1 == stat(dirbuf, &statbuf)) {
-		if (-1 == my_mkdir(dirbuf, (mode_t) (S_IRWXU)))
+	if (stat(dirbuf, &statbuf) == -1) {
+		if (my_mkdir(dirbuf, (mode_t) (S_IRWXU)) == -1)
 			/* Can't create directory: Fall back on Homedir */
 			joinpath(filebuf, sizeof(filebuf), homedir, OLDNEWSRC_FILE);
 	}
@@ -419,15 +419,19 @@ group_get_art_info(
 
 		snprintf(line, sizeof(line), "GROUP %s", groupname);
 #	ifdef DEBUG
-		if (debug & DEBUG_NNTP)
+		if ((debug & DEBUG_NNTP) && verbose > 1)
 			debug_print_file("NNTP", "group_get_art_info %s", line);
 #	endif /* DEBUG */
 		put_server(line);
 
 		switch (get_respcode(line, sizeof(line))) {
 			case OK_GROUP:
-				if (sscanf(line, "%"T_ARTNUM_SFMT" %"T_ARTNUM_SFMT" %"T_ARTNUM_SFMT, art_count, art_min, art_max) != 3)
-					error_message(2, _(txt_error_invalid_response_to_group), line);
+				if (sscanf(line, "%"T_ARTNUM_SFMT" %"T_ARTNUM_SFMT" %"T_ARTNUM_SFMT, art_count, art_min, art_max) != 3) {
+#	ifdef DEBUG
+					if ((debug & DEBUG_NNTP) && verbose > 1)
+						debug_print_file("NNTP", "Invalid response to GROUP command, %s", line);
+#	endif /* DEBUG */
+				}
 				break;
 
 			case ERR_NOGROUP:
@@ -444,7 +448,7 @@ group_get_art_info(
 
 			default:
 #	ifdef DEBUG
-				if (debug & DEBUG_NNTP)
+				if ((debug & DEBUG_NNTP) && verbose > 1)
 					debug_print_file("NNTP", "NOT_OK %s", line);
 #	endif /* DEBUG */
 				return -1;
@@ -1231,7 +1235,7 @@ pos_group_in_newsrc(
 
 	while ((line = tin_fgets(fp_in, FALSE)) != NULL) {
 		if (STRNCMPEQ(group->name, line, group_len) && line[group_len] == SUBSCRIBED) {
-			FreeAndNull(newsgroup);
+			FreeIfNeeded(newsgroup);
 			newsgroup = my_strdup(line);		/* Take a copy of this line */
 			found = TRUE;
 			continue;
@@ -1301,10 +1305,8 @@ pos_group_in_newsrc(
 		subscribed_pos++;
 	}
 
-	if (!repositioned) {
+	if (!repositioned)
 		write_newsrc_line(fp_out, newsgroup);
-		repositioned = TRUE;
-	}
 
 	/*
 	 * append unsubscribed groups file to newnewsrc
@@ -1325,10 +1327,8 @@ pos_group_in_newsrc(
 			fclose(fp_out);
 		}
 	} else {
-		if (repositioned) {
-			rename_file(newnewsrc, newsrc);
-			ret_code = TRUE;
-		}
+		rename_file(newnewsrc, newsrc);
+		ret_code = TRUE;
 	}
 	fp_out = NULL;
 	newnewsrc_created = FALSE;
