@@ -3,11 +3,11 @@
  *  Module    : tcurses.c
  *  Author    : Thomas Dickey <dickey@invisible-island.net>
  *  Created   : 1997-03-02
- *  Updated   : 2017-10-18
+ *  Updated   : 2018-07-02
  *  Notes     : This is a set of wrapper functions adapting the termcap
  *	             interface of tin to use SVr4 curses (e.g., ncurses).
  *
- * Copyright (c) 1997-2018 Thomas Dickey <dickey@invisible-island.net>
+ * Copyright (c) 1997-2019 Thomas Dickey <dickey@invisible-island.net>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -61,9 +61,9 @@ int cCOLS;
 static int my_innstr(char *str, int n);
 
 
-#	if defined(HAVE_XCURSES) && !defined(HAVE_VWPRINTW)
+#	if defined(HAVE_XCURSES) && !defined(HAVE_VW_PRINTW)
 static int
-vwprintw(
+vw_printw(
 	WINDOW *w,
 	char *fmt,
 	va_list ap)
@@ -74,7 +74,7 @@ vwprintw(
 
 	vsnprintf(buffer, sizeof(buffer), fmt, ap);
 	getyx(w, y, x);
-	TRACE(("vwprintw[%d/%d,%d/%d]:%s", y, cLINES, x, cCOLS, buffer));
+	TRACE(("vw_printw[%d/%d,%d/%d]:%s", y, cLINES, x, cCOLS, buffer));
 	while (*string == '\b')
 		string++;
 	code = waddstr(w, string);
@@ -84,7 +84,7 @@ vwprintw(
 	}
 	return code;
 }
-#	endif /* HAVE_XCURSES && !HAVE_VWPRINTW */
+#	endif /* HAVE_XCURSES && !HAVE_VW_PRINTW */
 
 
 /*
@@ -657,7 +657,7 @@ my_printf(
 		if (flag)
 			Raw(TRUE);
 	} else
-		vwprintw(stdscr, fmt, ap);
+		vw_printw(stdscr, fmt, ap);
 
 	va_end(ap);
 }
@@ -681,7 +681,7 @@ my_fprintf(
 		if (flag)
 			Raw(TRUE);
 	} else
-		vwprintw(stdscr, fmt, ap);
+		vw_printw(stdscr, fmt, ap);
 
 	va_end(ap);
 }
@@ -843,7 +843,7 @@ my_retouch(
 
 /*
  * innstr can't read multibyte chars
- * we use innwstr (if avaible) and convert to multibyte chars
+ * we use innwstr (if available) and convert to multibyte chars
  */
 static int
 my_innstr(
