@@ -1,10 +1,10 @@
 #! /usr/bin/perl -w
 #
-# reads an article on STDIN, mails any copies if requied,
+# reads an article on STDIN, mails any copies if required,
 # signs the article and posts it.
 #
 #
-# Copyright (c) 2002-2017 Urs Janssen <urs@tin.org>,
+# Copyright (c) 2002-2019 Urs Janssen <urs@tin.org>,
 #                         Marc Brockschmidt <marc@marcbrockschmidt.de>
 #
 # Redistribution and use in source and binary forms, with or without
@@ -32,22 +32,25 @@
 # POSSIBILITY OF SUCH DAMAGE.
 #
 #
-# TODO: - add debug mode which doesn't delete tmp-files and is verbose
+# TODO: - extend debug mode to not delete tmp-files and be more verbose
 #       - add pid to pgptmpf to allow multiple simultaneous instances
 #       - check for /etc/nntpserver (and /etc/news/server)
-#       - add $NEWSHOST fallback for $NNTPSERVER (like in Net::NNTP)?
 #       - add $PGPOPTS, $PGPPATH and $GNUPGHOME support
 #       - cleanup and remove duplicated code
 #       - option to convert CRLF to LF in input
-#       - use STARTTLS (if Net::NNTP is recent enough and server supports it)
+#       - use STARTTLS (if Net::NNTP is recent enough and server supports it)?
 #       - quote inpupt properly before passing to shell
+#       - if (!defined $ENV{'GPG_TTY'}) {if (open(my $T,'-|','tty')) {
+#           chomp(my $tty=<$T>); close($T);
+#           $ENV{'GPG_TTY'}=$tty if($tty =~ m/^\//)}}
+#         for gpg?
 #
 
 use strict;
 use warnings;
 
 # version Number
-my $version = "1.1.42";
+my $version = "1.1.50";
 
 my %config;
 
@@ -80,9 +83,10 @@ $config{'pgpheader'}	= 'X-PGP-Sig';
 $config{'pgpbegin'}		= '-----BEGIN PGP SIGNATURE-----';	# Begin of PGP-Signature
 $config{'pgpend'}		= '-----END PGP SIGNATURE-----';	# End of PGP-Signature
 
+$config{'canlock_algorithm'}	= 'sha1'; 	# Digest algorithm used for cancel-lock and cancel-key; sha1, sha256 and sha512 are supported
 # $config{'canlock_secret'}	= '~/.cancelsecret';		# Path to canlock secret file
 
-# $config{'ignore_headers'} = '';      # headers to be ignored during signing
+# $config{'ignore_headers'} = '';		# headers to be ignored during signing
 
 $config{'PGPSignHeaders'} = ['From', 'Newsgroups', 'Subject', 'Control',
 	'Supersedes', 'Followup-To', 'Date', 'Injection-Date', 'Sender', 'Approved',
@@ -130,7 +134,8 @@ if (defined($TINEWSRC)) {
 # digest-algo is case sensitive and should be all uppercase
 $config{'digest-algo'} = uc($config{'digest-algo'});
 
-# these env-vars have higher priority
+# these env-vars have higher priority (order is important)
+$config{'NNTPServer'} = $ENV{'NEWSHOST'} if ($ENV{'NEWSHOST'});
 $config{'NNTPServer'} = $ENV{'NNTPSERVER'} if ($ENV{'NNTPSERVER'});
 $config{'NNTPPort'} = $ENV{'NNTPPORT'} if ($ENV{'NNTPPORT'});
 
@@ -148,6 +153,7 @@ GetOptions('A|V|W|O|no-organization|h|headers' => [], # do nothing
 	'force-auth|Y'	=> \$config{'force_auth'},
 	'approved|a=s'	=> \$config{'approved'},
 	'control|c=s'	=> \$config{'control'},
+	'canlock-algorithm=s'	=> \$config{'canlock_algorithm'},
 	'distribution|d=s'	=> \$config{'distribution'},
 	'expires|e=s'	=> \$config{'expires'},
 	'from|f=s'	=> \$config{'from'},
@@ -179,19 +185,45 @@ usage() if ($config{'help'});
 my $sha_mod=undef;
 # Cancel-Locks require some more modules
 if ($config{'canlock_secret'} && !$config{'no_canlock'}) {
-	foreach ('Digest::SHA qw(sha1)', 'Digest::SHA1()') {
-		eval "use $_";
-		if (!$@) {
-			($sha_mod = $_) =~ s#( qw\(sha1\)|\(\))##;
-			last;
-		}
+	$config{'canlock_algorithm'} = lc($config{'canlock_algorithm'});
+	# we support sha1, sha256 and sha512, fallback to sha1 if something else is given
+	if (!($config{'canlock_algorithm'} =~ /^sha(1|256|512)$/)) {
+		warn "Digest algorithm " . $config{'canlock_algorithm'} . " not supported. Falling back to sha1.\n" if $config{'debug'};
+		$config{'canlock_algorithm'} = 'sha1';
 	}
-	foreach ('MIME::Base64()', 'Digest::HMAC_SHA1()') {
-		eval "use $_";
-		if ($@ || !defined($sha_mod)) {
-			$config{'no_canlock'} = 1;
-			warn "Cancel-Locks disabled: Can't locate ".$_."\n" if $config{'debug'};
-			last;
+	if ($config{'canlock_algorithm'} eq 'sha1') {
+		foreach ('Digest::SHA qw(sha1)', 'Digest::SHA1()') {
+			eval "use $_";
+			if (!$@) {
+				($sha_mod = $_) =~ s#( qw\(sha1\)|\(\))##;
+				last;
+			}
+		}
+		foreach ('MIME::Base64()', 'Digest::HMAC_SHA1()') {
+			eval "use $_";
+			if ($@ || !defined($sha_mod)) {
+				$config{'no_canlock'} = 1;
+				warn "Cancel-Locks disabled: Can't locate ".$_."\n" if $config{'debug'};
+				last;
+			}
+		}
+	} elsif ($config{'canlock_algorithm'} eq 'sha256') {
+		foreach ('MIME::Base64()', 'Digest::SHA qw(sha256 hmac_sha256)') {
+			eval "use $_";
+			if ($@) {
+	 			$config{'no_canlock'} = 1;
+				warn "Cancel-Locks disabled: Can't locate ".$_."\n" if $config{'debug'};
+				last;
+			}
+		}
+	} else {
+		foreach ('MIME::Base64()', 'Digest::SHA qw(sha512 hmac_sha512)') {
+			eval "use $_";
+			if ($@) {
+	 			$config{'no_canlock'} = 1;
+				warn "Cancel-Locks disabled: Can't locate ".$_."\n" if $config{'debug'};
+				last;
+			}
 		}
 	}
 }
@@ -228,11 +260,11 @@ readarticle(\%Header, \@Body);
 # Add signature if there is none
 if (!$config{'no_signature'}) {
 	if ($config{'add_signature'} && !grep {/^-- /} @Body) {
-		if (-r $config{'sig_path'}) {
+		if (-r glob($config{'sig_path'})) {
 			my $l = 0;
 			push @Body, "-- \n";
-			open(my $SIGNATURE, '<', $config{'sig_path'}) or die("Can't open " . $config{'sig_path'} . ": $!");
-			while (<$SIGNATURE>){
+			open(my $SIGNATURE, '<', glob($config{'sig_path'})) or die("Can't open " . $config{'sig_path'} . ": $!");
+			while (<$SIGNATURE>) {
 				die $config{'sig_path'} . " longer than " . $config{'sig_max_lines'}. " lines!" if (++$l > $config{'sig_max_lines'});
 				push @Body, $_;
 			}
@@ -368,44 +400,38 @@ if ($config{'canlock_secret'} && !$config{'no_canlock'} && defined($Header{'mess
 	close($CANLock);
 	(my $data = $Header{'message-id'}) =~ s#^Message-ID: ##i;
 	chomp $data;
-	my $digest = Digest::HMAC_SHA1::hmac_sha1($data, $key);
-	my $cancel_key = MIME::Base64::encode($digest, '');
-	my $cancel_lock;
-	if ($sha_mod =~ m/SHA1/) {
-		$cancel_lock = MIME::Base64::encode(Digest::SHA1::sha1($cancel_key, ''));
-	} else {
-		$cancel_lock = MIME::Base64::encode(Digest::SHA::sha1($cancel_key, ''));
-	}
+	my $cancel_key = buildcancelkey($data, $key);
+	my $cancel_lock = buildcancellock($cancel_key, $sha_mod);
 	if (defined($Header{'cancel-lock'})) {
 		chomp $Header{'cancel-lock'};
-		$Header{'cancel-lock'} .= " sha1:" . $cancel_lock;
+		$Header{'cancel-lock'} .= " " . $config{'canlock_algorithm'} . ":" . $cancel_lock . "\n";
 	} else {
-		$Header{'cancel-lock'} = "Cancel-Lock: sha1:" . $cancel_lock;
+		$Header{'cancel-lock'} = "Cancel-Lock: " . $config{'canlock_algorithm'} . ":" . $cancel_lock . "\n";
 	}
 
 	if ((defined($Header{'supersedes'}) && $Header{'supersedes'} =~ m/^Supersedes:\s+<\S+>\s*$/i) || (defined($Header{'control'}) && $Header{'control'} =~ m/^Control:\s+cancel\s+<\S+>\s*$/i) ||(defined($Header{'also-control'}) && $Header{'also-control'} =~ m/^Also-Control:\s+cancel\s+<\S+>\s*$/i)) {
 		if (defined($Header{'also-control'}) && $Header{'also-control'} =~ m/^Also-Control:\s+cancel\s+/i) {
 			($data = $Header{'also-control'}) =~ s#^Also-Control:\s+cancel\s+##i;
 			chomp $data;
-			$cancel_key = MIME::Base64::encode(Digest::HMAC_SHA1::hmac_sha1($data, $key),'');
+			$cancel_key = buildcancelkey($data, $key);
 		} else {
 			if (defined($Header{'control'}) && $Header{'control'} =~ m/^Control: cancel /i) {
 				($data = $Header{'control'})=~ s#^Control:\s+cancel\s+##i;
 				chomp $data;
-				$cancel_key = MIME::Base64::encode(Digest::HMAC_SHA1::hmac_sha1($data, $key),'');
+				$cancel_key = buildcancelkey($data, $key);
 			} else {
 				if (defined($Header{'supersedes'})) {
 					($data = $Header{'supersedes'}) =~ s#^Supersedes: ##i;
 					chomp $data;
-					$cancel_key = MIME::Base64::encode(Digest::HMAC_SHA1::hmac_sha1($data, $key),'');
+					$cancel_key = buildcancelkey($data, $key);
 				}
 			}
 		}
 		if (defined($Header{'cancel-key'})) {
 			chomp $Header{'cancel-key'};
-			$Header{'cancel-key'} .= " sha1:" . $cancel_key . "\n";
+			$Header{'cancel-key'} .= " " . $config{'canlock_algorithm'} . ":" . $cancel_key . "\n";
 		} else {
-			$Header{'cancel-key'} = "Cancel-Key: sha1:" . $cancel_key . "\n";
+			$Header{'cancel-key'} = "Cancel-Key: " . $config{'canlock_algorithm'} . ":" . $cancel_key . "\n";
 		}
 	}
 }
@@ -530,7 +556,7 @@ sub getdate {
 # are defined, the sub will try to ask the user (only if
 # $config{'Interactive'} is != 0).
 sub AuthonNNTP {
-	my $Server = Net::NNTP->new($config{'NNTPServer'}, Reader => 1, Debug => 0, Port => $config{'NNTPPort'})
+	my $Server = Net::NNTP->new($config{'NNTPServer'}, Reader => 1, Debug => $config{'debug'}, Port => $config{'NNTPPort'})
 		or die("$0: Can't connect to ".$config{'NNTPServer'}.":".$config{'NNTPPort'}."!\n");
 	my $ServerMsg = "";
 	my $ServerCod = $Server->code();
@@ -700,7 +726,7 @@ sub savearticle {
 
 #-------- sub signarticle
 # signarticle signs an article and returns a reference to an array
-# 	containing the whole signed Message.
+# containing the whole signed Message.
 #
 # Receives:
 # 	- $HeaderR: A reference to a hash containing the articles headers.
@@ -747,6 +773,7 @@ sub signarticle {
 	unless (m/\Q$config{'pgpbegin'}\E$/o) {
 		unlink $config{'pgptmpf'} . ".txt";
 		unlink $config{'pgptmpf'} . ".txt.asc";
+		close($FH);
 		die("$0: ".$config{'pgpbegin'}." not found in ".$config{'pgptmpf'}.".txt.asc\n");
 	}
 	unlink($config{'pgptmpf'} . ".txt") or warn "$0: Couldn't unlink ".$config{'pgptmpf'}.".txt: $!\n";
@@ -755,6 +782,7 @@ sub signarticle {
 	$_ = <$FH>;
 	unless (m/^Version: (\S+)(?:\s(\S+))?/o) {
 		unlink $config{'pgptmpf'} . ".txt.asc";
+		close($FH);
 		die("$0: didn't find PGP Version line where expected.\n");
 	}
 	if (defined($2)) {
@@ -776,6 +804,7 @@ sub signarticle {
 	$_ = <$FH>;
 	unless (eof($FH)) {
 		unlink $config{'pgptmpf'} . ".txt.asc";
+		close($FH);
 		die("$0: unexpected data following ".$config{'pgpend'}."\n");
 	}
 	close($FH);
@@ -808,6 +837,55 @@ sub signarticle {
 	return \@pgpmessage;
 }
 
+#-------- sub buildcancelkey
+# buildcancelkey builds the cancel-key based on the configured HASH algorithm.
+#
+# Receives:
+# 	- $data: The input data.
+# 	- $key: The secret key to be used.
+#
+# Returns:
+# 	- $cancel_key: The calculated cancel-key.
+sub buildcancelkey {
+	my ($data, $key) = @_;
+	my $cancel_key;
+	if ($config{'canlock_algorithm'} eq 'sha1') {
+		$cancel_key = MIME::Base64::encode(Digest::HMAC_SHA1::hmac_sha1($data, $key), '');
+	} elsif ($config{'canlock_algorithm'} eq 'sha256') {
+		$cancel_key = MIME::Base64::encode(Digest::SHA::hmac_sha256($data, $key), '');
+	} else {
+		$cancel_key = MIME::Base64::encode(Digest::SHA::hmac_sha512($data, $key), '');
+	}
+	return $cancel_key;
+}
+
+#-------- sub buildcancellock
+# buildcancellock builds the cancel-lock based on the configured HASH algorithm
+# and the given cancel-key.
+#
+# Receives:
+# 	- $sha_mod: A hint which module to be used for sha1.
+# 	- $cancel_key: The cancel-key for which the lock has to be calculated.
+#
+# Returns:
+# 	- $cancel_lock: The calculated cancel-lock.
+sub buildcancellock {
+	my ($cancel_key, $sha_mod) = @_;
+	my $cancel_lock;
+	if ($config{'canlock_algorithm'} eq 'sha1') {
+		if ($sha_mod =~ m/SHA1/) {
+			$cancel_lock = MIME::Base64::encode(Digest::SHA1::sha1($cancel_key, ''), '');
+		} else {
+			$cancel_lock = MIME::Base64::encode(Digest::SHA::sha1($cancel_key, ''), '');
+		}
+	} elsif ($config{'canlock_algorithm'} eq 'sha256') {
+		$cancel_lock = MIME::Base64::encode(Digest::SHA::sha256($cancel_key, ''), '');
+	} else {
+		$cancel_lock = MIME::Base64::encode(Digest::SHA::sha512($cancel_key, ''), '');
+	}
+	return $cancel_lock;
+}
+
 sub version {
 	print $pname." ".$version."\n";
 	return;
@@ -831,6 +909,8 @@ sub usage {
 	print "  -v         show version\n";
 	print "  -w string  set Followup-To:-header to string\n";
 	print "  -x string  set Path:-header to string\n";
+	print "  -D         enable debugging\n";
+	print "  -F string  set References:-header to string\n";
 	print "  -H         show help\n";
 	print "  -I         do not add Injection-Date: header\n";
 	print "  -L         do not add Cancel-Lock: / Cancel-Key: headers\n";
@@ -853,8 +933,12 @@ B<tinews.pl> [B<OPTIONS>] E<lt> I<input>
 
 =head1 DESCRIPTION
 
-B<tinews.pl> reads an article on STDIN, signs it via B<pgp>(1) or
-B<gpg>(1) and posts it to a news server.
+B<tinews.pl> reads an article on STDIN, signs it via L<pgp(1)> or
+L<gpg(1)> and posts it to a news server.
+
+The article shall not contain any raw 8-bit data or it needs to
+already have the relevant MIME-headers as B<tinews.pl> will not
+add any MIME-headers nor encode its input.
 
 If the article contains To:, Cc: or Bcc: headers and mail-actions are
 configured it will automatically add a "Posted-And-Mailed: yes" header
@@ -866,7 +950,7 @@ Cancel-Lock: (and Cancel-Key: if required) header.
 The input should have unix line endings (<LF>, '\n').
 
 =head1 OPTIONS
-X<tinews, commandline options>
+X<tinews, command-line options>
 
 =over 4
 
@@ -969,6 +1053,12 @@ X<-L> X<--no-canlock>
 
 Do not add Cancel-Lock: / Cancel-Key: headers.
 
+=item --B<canlock-algorithm> C<Algorithm>
+X<--canlock-algorithm>
+
+Digest algorithm used for Cancel-Lock: / Cancel-Key: headers.
+Supported algorithms are sha1, sha256 and sha512. Default is sha1.
+
 =item -B<R> | --B<no-control>
 X<-R> X<--no-control>
 
@@ -1007,7 +1097,9 @@ These options are accepted for compatibility reasons but ignored.
 =item -B<D> | -B<N> | --B<debug>
 X<-D> X<-N> X<--debug>
 
-These options are accepted but do not have any functionality yet.
+Enable warnings about raw 8-bit data and set L<Net::NNTP(3pm)> in debug
+mode, enable warnings about raw 8-bit data, warn about disabled options
+due to lacking perl-modules or executables and unreadable files.
 
 =back
 
@@ -1032,10 +1124,17 @@ X<tinews, environment variables>
 
 =over 4
 
+=item B<$NEWSHOST>
+X<$NEWSHOST> X<NEWSHOST>
+
+Set to override the NNTP server configured in the source or config-file.
+It has lower priority than B<$NNTPSERVER> and should be avoided.
+
 =item B<$NNTPSERVER>
 X<$NNTPSERVER> X<NNTPSERVER>
 
-Set to override the NNTP server configured in the source.
+Set to override the NNTP server configured in the source or config-file.
+This has higher priority than B<$NEWSHOST>.
 
 =item B<$NNTPPORT>
 X<$NNTPPORT> X<NNTPPORT>
@@ -1048,12 +1147,12 @@ B<$NNTPPORT>.
 X<$PGPPASS> X<PGPPASS>
 
 Set to override the passphrase configured in the source (used for
-B<pgp>(1)-2.6.3).
+L<pgp(1)>-2.6.3).
 
 =item B<$PGPPASSFILE>
 X<$PGPPASSFILE> X<PGPPASSFILE>
 
-Passphrase file used for B<pgp>(1) or B<gpg>(1).
+Passphrase file used for L<pgp(1)> or L<gpg(1)>.
 
 =item B<$SIGNER>
 X<$SIGNER> X<SIGNER>
@@ -1073,7 +1172,7 @@ The '-B<r>' command-line option overrides B<$REPLYTO>.
 X<$ORGANIZATION> X<ORGANIZATION>
 
 Set the article header field Organization: to the contents of the variable
-if there isn't already a Organization: header in the article. The '-B<o>'
+if there isn't already an Organization: header in the article. The '-B<o>'
 command-line option overrides B<$ORGANIZATION>.
 
 =item B<$DISTRIBUTION>
@@ -1099,7 +1198,7 @@ Temporary file used to store the reformatted and signed article.
 
 =item F<$PGPPASSFILE>
 
-The passphrase file to be used for B<pgp>(1) or B<gpg>(1).
+The passphrase file to be used for L<pgp(1)> or L<gpg(1)>.
 
 =item F<$HOME/.signature>
 
@@ -1143,16 +1242,27 @@ security is an issue, don't use this script.
 
 =head1 NOTES
 
-B<tinews.pl> is designed to be used with B<pgp>(1)-2.6.3,
-B<pgp>(1)-5, B<pgp>(1)-6, B<gpg>(1) and B<gpg2>(1).
+B<tinews.pl> is designed to be used with L<pgp(1)>-2.6.3,
+L<pgp(1)>-5, L<pgp(1)>-6, L<gpg(1)> and L<gpg2(1)>.
 
 B<tinews.pl> requires the following standard modules to be installed:
-B<Getopt::Long>(3pm), B<Net::NNTP>(3pm), B<Time::Local>(3pm) and
-B<Term::Readline>(3pm).
+L<Getopt::Long(3pm)>, L<Net::NNTP(3pm)>, <Time::Local(3pm)> and
+L<Term::Readline(3pm)>.
 
-If the Cancel-Lock feature is enabled the following additional modules
-must be installed: B<MIME::Base64>(3pm), B<Digest::SHA>(3pm) or
-B<Digest::SHA1>(3pm) and B<Digest::HMAC_SHA1>(3pm)
+If the Cancel-Lock feature (RFC 8315) is enabled the following additional
+modules must be installed: L<MIME::Base64(3pm)>, L<Digest::SHA(3pm)> or
+L<Digest::SHA1(3pm)> and L<Digest::HMAC_SHA1(3pm)>. sha256 and sha512 as
+algorithms for B<canlock-algorithm> are only available with L<Digest::SHA(3pm)>.
+
+L<gpg2(1)> users may need to set B<$GPG_TTY>, i.e.
+
+ GPG_TTY=$(tty)
+ export GPG_TTY
+
+before using B<tinews.pl>. See L<https://www.gnupg.org/> for details.
+
+B<tinews.pl> does not do any MIME encoding, its input should be already
+properly encoded and have all relevant headers set.
 
 =head1 AUTHOR
 
@@ -1161,9 +1271,9 @@ Marc Brockschmidt E<lt>marc@marcbrockschmidt.deE<gt>
 
 =head1 SEE ALSO
 
-B<pgp>(1), B<gpg>(1), B<gpg2>(1), B<pgps>(1), B<Digest::HMAC_SHA1>(3pm),
-B<Digest::SHA>(3pm), B<Digest::SHA1>(3pm), B<Getopt::Long>(3pm),
-B<MIME::Base64>(3pm), B<Net::NNTP>(3pm), B<Time::Local>(3pm),
-B<Term::Readline>(3pm)
+L<pgp(1)>, L<gpg(1)>, L<gpg2(1)>, L<pgps(1)>, L<Digest::HMAC_SHA1(3pm)>,
+L<Digest::SHA(3pm)>, L<Digest::SHA1(3pm)>, L<Getopt::Long(3pm)>,
+L<MIME::Base64(3pm)>, L<Net::NNTP(3pm)>, L<Time::Local(3pm)>,
+L<Term::Readline(3pm)>
 
 =cut
